@@ -20,6 +20,9 @@
 #
 # Urwid web site: http://excess.org/urwid/
 
+from __future__ import division, print_function
+
+from urwid.compat import with_metaclass
 from urwid.util import decompose_tagmarkup, get_encoding_mode
 from urwid.canvas import CompositeCanvas, CanvasJoin, TextCanvas, \
     CanvasCombine, SolidCanvas
@@ -96,7 +99,7 @@ class BigText(Widget):
 
 class LineBox(WidgetDecoration, WidgetWrap):
 
-    def __init__(self, original_widget, title="",
+    def __init__(self, original_widget, title="", title_align="center",
                  tlcorner=u'┌', tline=u'─', lline=u'│',
                  trcorner=u'┐', blcorner=u'└', rline=u'│',
                  bline=u'─', brcorner=u'┘'):
@@ -106,6 +109,9 @@ class LineBox(WidgetDecoration, WidgetWrap):
         Use 'title' to set an initial title text with will be centered
         on top of the box.
 
+        Use `title_align` to align the title to the 'left', 'right', or 'center'.
+        The default is 'center'.
+
         You can also override the widgets used for the lines/corners:
             tline: top line
             bline: bottom line
@@ -116,37 +122,77 @@ class LineBox(WidgetDecoration, WidgetWrap):
             blcorner: bottom left corner
             brcorner: bottom right corner
 
+        If empty string is specified for one of the lines/corners, then no
+        character will be output there.  This allows for seamless use of
+        adjoining LineBoxes.
         """
 
-        tline, bline = Divider(tline), Divider(bline)
-        lline, rline = SolidFill(lline), SolidFill(rline)
+        if tline:
+            tline = Divider(tline)
+        if bline:
+            bline = Divider(bline)
+        if lline:
+            lline = SolidFill(lline)
+        if rline:
+            rline = SolidFill(rline)
         tlcorner, trcorner = Text(tlcorner), Text(trcorner)
         blcorner, brcorner = Text(blcorner), Text(brcorner)
 
+        if not tline and title:
+            raise ValueError('Cannot have a title when tline is empty string')
+
         self.title_widget = Text(self.format_title(title))
-        self.tline_widget = Columns([
-            tline,
-            ('flow', self.title_widget),
-            tline,
-        ])
 
-        top = Columns([
-            ('fixed', 1, tlcorner),
-            self.tline_widget,
-            ('fixed', 1, trcorner)
-        ])
+        if tline:
+            if title_align not in ('left', 'center', 'right'):
+                raise ValueError('title_align must be one of "left", "right", or "center"')
+            if title_align == 'left':
+                tline_widgets = [('flow', self.title_widget), tline]
+            else:
+                tline_widgets = [tline, ('flow', self.title_widget)]
+                if title_align == 'center':
+                    tline_widgets.append(tline)
+            self.tline_widget = Columns(tline_widgets)
+            top = Columns([
+                ('fixed', 1, tlcorner),
+                self.tline_widget,
+                ('fixed', 1, trcorner)
+            ])
 
-        middle = Columns([
-            ('fixed', 1, lline),
-            original_widget,
-            ('fixed', 1, rline),
-        ], box_columns=[0, 2], focus_column=1)
+        else:
+            self.tline_widget = None
+            top = None
 
-        bottom = Columns([
-            ('fixed', 1, blcorner), bline, ('fixed', 1, brcorner)
-        ])
+        middle_widgets = []
+        if lline:
+            middle_widgets.append(('fixed', 1, lline))
+        else:
+            # Note: We need to define a fixed first widget (even if it's 0 width) so that the other
+            # widgets have something to anchor onto
+            middle_widgets.append(('fixed', 0, SolidFill(u"")))
+        middle_widgets.append(original_widget)
+        focus_col = len(middle_widgets) - 1
+        if rline:
+            middle_widgets.append(('fixed', 1, rline))
 
-        pile = Pile([('flow', top), middle, ('flow', bottom)], focus_item=1)
+        middle = Columns(middle_widgets,
+                box_columns=[0, 2], focus_column=focus_col)
+
+        if bline:
+            bottom = Columns([
+                ('fixed', 1, blcorner), bline, ('fixed', 1, brcorner)
+            ])
+        else:
+            bottom = None
+
+        pile_widgets = []
+        if top:
+            pile_widgets.append(('flow', top))
+        pile_widgets.append(middle)
+        focus_pos = len(pile_widgets) - 1
+        if bottom:
+            pile_widgets.append(('flow', bottom))
+        pile = Pile(pile_widgets, focus_item=focus_pos)
 
         WidgetDecoration.__init__(self, original_widget)
         WidgetWrap.__init__(self, pile)
@@ -158,6 +204,8 @@ class LineBox(WidgetDecoration, WidgetWrap):
             return ""
 
     def set_title(self, text):
+        if not self.title_widget:
+            raise ValueError('Cannot set title when tline is unset')
         self.title_widget.set_text(self.format_title(text))
         self.tline_widget._invalidate()
 
@@ -192,9 +240,7 @@ def nocache_bargraph_get_data(self, get_data_fn):
 class BarGraphError(Exception):
     pass
 
-class BarGraph(Widget):
-    __metaclass__ = BarGraphMeta
-
+class BarGraph(with_metaclass(BarGraphMeta, Widget)):
     _sizing = frozenset([BOX])
 
     ignore_focus = True
@@ -481,7 +527,8 @@ class BarGraph(Widget):
         o = []
         r = 0  # row remainder
 
-        def seg_combine((bt1, w1), (bt2, w2)):
+        def seg_combine(a, b):
+            (bt1, w1), (bt2, w2) = a, b
             if (bt1, w1) == (bt2, w2):
                 return (bt1, w1), None, None
             wmin = min(w1, w2)
@@ -811,6 +858,28 @@ class ProgressBar(Widget):
                      foreground of satt corresponds to the normal part and the
                      background corresponds to the complete part.  If satt
                      is ``None`` then no smoothing will be done.
+
+        >>> pb = ProgressBar('a', 'b')
+        >>> pb
+        <ProgressBar flow widget>
+        >>> print(pb.get_text())
+        0 %
+        >>> pb.set_completion(34.42)
+        >>> print(pb.get_text())
+        34 %
+        >>> class CustomProgressBar(ProgressBar):
+        ...     def get_text(self):
+        ...         return u'Foobar'
+        >>> cpb = CustomProgressBar('a', 'b')
+        >>> print(cpb.get_text())
+        Foobar
+        >>> for x in range(101):
+        ...     cpb.set_completion(x)
+        ...     s = cpb.render((10, ))
+        >>> cpb2 = CustomProgressBar('a', 'b', satt='c')
+        >>> for x in range(101):
+        ...     cpb2.set_completion(x)
+        ...     s = cpb2.render((10, ))
         """
         self.normal = normal
         self.complete = complete
@@ -840,6 +909,7 @@ class ProgressBar(Widget):
     def get_text(self):
         """
         Return the progress bar percentage text.
+        You can override this method to display custom text.
         """
         percent = min(100, max(0, int(self.current * 100 / self.done)))
         return str(percent) + " %"
@@ -853,7 +923,12 @@ class ProgressBar(Widget):
         c = txt.render((maxcol,))
 
         cf = float(self.current) * maxcol / self.done
-        ccol = int(cf)
+        ccol_dirty = int(cf)
+        ccol = len(c._text[0][:ccol_dirty].decode(
+            'utf-8', 'ignore'
+        ).encode(
+            'utf-8'
+        ))
         cs = 0
         if self.satt is not None:
             cs = int((cf - ccol) * 8)
@@ -909,3 +984,10 @@ class PythonLogo(Widget):
         """
         fixed_size(size)
         return self._canvas
+
+def _test():
+    import doctest
+    doctest.testmod()
+
+if __name__=='__main__':
+    _test()
